package org.apache.bcel.util;

import java.io.File;
import java.nio.file.DirectoryStream;
import java.util.function.Consumer;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.io.IOException;
import java.nio.file.Path;
import java.util.Map;
import java.net.URL;
import java.nio.file.Paths;
import java.util.Collections;
import java.nio.file.FileSystems;
import java.net.URI;
import java.nio.file.FileSystem;
import java.net.URLClassLoader;
import java.io.Closeable;

public class ModularRuntimeImage implements Closeable
{
    static final String MODULES_PATH;
    static final String PACKAGES_PATH;
    private final URLClassLoader classLoader;
    private final FileSystem fileSystem;
    
    public ModularRuntimeImage() {
        this(null, FileSystems.getFileSystem(URI.create("jrt:/")));
    }
    
    public ModularRuntimeImage(final String javaHome) throws IOException {
        final Map<String, ?> emptyMap = Collections.emptyMap();
        final Path jrePath = Paths.get(javaHome, new String[0]);
        final Path jrtFsPath = jrePath.resolve("lib").resolve("jrt-fs.jar");
        this.classLoader = URLClassLoader.newInstance(new URL[] { jrtFsPath.toUri().toURL() });
        this.fileSystem = FileSystems.newFileSystem(URI.create("jrt:/"), emptyMap, this.classLoader);
    }
    
    private ModularRuntimeImage(final URLClassLoader cl, final FileSystem fs) {
        this.classLoader = cl;
        this.fileSystem = fs;
    }
    
    @Override
    public void close() throws IOException {
        if (this.classLoader != null) {
            this.classLoader.close();
        }
        if (this.fileSystem != null) {
            this.fileSystem.close();
        }
    }
    
    public FileSystem getFileSystem() {
        return this.fileSystem;
    }
    
    public List<Path> list(final Path dirPath) throws IOException {
        final List<Path> list = new ArrayList<Path>();
        try (DirectoryStream<Path> ds = Files.newDirectoryStream(dirPath)) {
            ds.forEach(list::add);
        }
        return list;
    }
    
    public List<Path> list(final String dirName) throws IOException {
        return this.list(this.fileSystem.getPath(dirName, new String[0]));
    }
    
    public List<Path> modules() throws IOException {
        return this.list(ModularRuntimeImage.MODULES_PATH);
    }
    
    public List<Path> packages() throws IOException {
        return this.list(ModularRuntimeImage.PACKAGES_PATH);
    }
    
    static {
        MODULES_PATH = File.separator + "modules";
        PACKAGES_PATH = File.separator + "packages";
    }
}
